"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
class FileCacheAdapter {
    constructor(options) {
        this.options = options;
    }
    async get(name) {
        const path = await this.path(name);
        if (!await fs_extra_1.pathExists(path)) {
            return null;
        }
        const payload = await fs_extra_1.readJSON(path);
        const modified = await this.getModifiedTime(payload.origin);
        if (!await fs_extra_1.pathExists(payload.origin) || payload.modified !== modified) {
            return null;
        }
        return payload.data;
    }
    async set(name, data, origin) {
        const modified = await this.getModifiedTime(origin);
        const path = await this.path(name);
        await fs_extra_1.writeJSON(path, { modified, data, origin });
    }
    async path(name) {
        await fs_extra_1.ensureDir(this.options.cacheDir);
        return `${this.options.cacheDir}/${name}.json`;
    }
    async getModifiedTime(origin) {
        const stats = await fs_extra_1.stat(origin);
        return stats.mtimeMs;
    }
}
exports.FileCacheAdapter = FileCacheAdapter;
