import { Configuration } from '../utils';
export declare abstract class Connection {
    protected readonly config: Configuration;
    protected readonly logger: import("../utils").Logger;
    protected readonly metadata: Record<string, import("../decorators").EntityMetadata<any>>;
    protected abstract client: any;
    constructor(config: Configuration);
    /**
     * Establishes connection to database
     */
    abstract connect(): Promise<void>;
    /**
     * Are we connected to the database
     */
    abstract isConnected(): Promise<boolean>;
    /**
     * Closes the database connection (aka disconnect)
     */
    abstract close(force?: boolean): Promise<void>;
    /**
     * Returns default client url for given driver (e.g. mongodb://127.0.0.1:27017 for mongodb)
     */
    abstract getDefaultClientUrl(): string;
    /**
     * Begins a transaction (if supported)
     */
    beginTransaction(savepoint?: string): Promise<void>;
    /**
     * Commits statements in a transaction
     */
    commit(savepoint?: string): Promise<void>;
    /**
     * Rollback changes in a transaction
     */
    rollback(savepoint?: string): Promise<void>;
    abstract execute(query: string, params?: any[], method?: 'all' | 'get' | 'run'): Promise<QueryResult | any | any[]>;
    getConnectionOptions(): ConnectionConfig;
    getClientUrl(): string;
    protected executeQuery<T>(query: string, params: any[], cb: () => Promise<T>): Promise<T>;
    protected logQuery(query: string, took: number): void;
}
export interface QueryResult {
    affectedRows: number;
    insertId: number;
    row?: Record<string, any>;
}
export interface ConnectionConfig {
    host?: string;
    port?: number;
    user?: string;
    password?: string;
    database?: string;
}
