"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const metadata_1 = require("../metadata");
class Connection {
    constructor(config) {
        this.config = config;
        this.logger = this.config.getLogger();
        this.metadata = metadata_1.MetadataStorage.getMetadata();
    }
    /**
     * Begins a transaction (if supported)
     */
    async beginTransaction(savepoint) {
        throw new Error(`Transactions are not supported by current driver`);
    }
    /**
     * Commits statements in a transaction
     */
    async commit(savepoint) {
        throw new Error(`Transactions are not supported by current driver`);
    }
    /**
     * Rollback changes in a transaction
     */
    async rollback(savepoint) {
        throw new Error(`Transactions are not supported by current driver`);
    }
    getConnectionOptions() {
        const ret = {};
        const url = new url_1.URL(this.config.getClientUrl());
        ret.host = this.config.get('host', url.hostname);
        ret.port = this.config.get('port', +url.port);
        ret.user = this.config.get('user', url.username);
        ret.password = this.config.get('password', url.password);
        ret.database = this.config.get('dbName', url.pathname.replace(/^\//, ''));
        return ret;
    }
    getClientUrl() {
        const options = this.getConnectionOptions();
        const url = new url_1.URL(this.config.getClientUrl(true));
        return `${url.protocol}//${options.user}${options.password ? ':*****' : ''}@${options.host}:${options.port}`;
    }
    async executeQuery(query, params, cb) {
        try {
            const now = Date.now();
            const res = await cb();
            this.logQuery(query, Date.now() - now);
            return res;
        }
        catch (e) {
            e.message += `\n in query: ${query}`;
            if (params && params.length) {
                e.message += `\n with params: ${JSON.stringify(params)}`;
            }
            throw e;
        }
    }
    logQuery(query, took) {
        this.logger.debug(`[query-logger] ${query} [took ${took} ms]`);
    }
}
exports.Connection = Connection;
