"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = require("mysql2/promise");
const fs_extra_1 = require("fs-extra");
const Connection_1 = require("./Connection");
class MySqlConnection extends Connection_1.Connection {
    async connect() {
        this.client = await promise_1.createConnection(this.getConnectionOptions());
    }
    async close(force) {
        await this.client.end({ force });
    }
    async isConnected() {
        try {
            await this.client.query('SELECT 1');
            return true;
        }
        catch (_a) {
            return false;
        }
    }
    async beginTransaction() {
        await this.query('START TRANSACTION');
    }
    async commit() {
        await this.query('COMMIT');
    }
    async rollback() {
        await this.query('ROLLBACK');
    }
    getDefaultClientUrl() {
        return 'mysql://root@127.0.0.1:3306';
    }
    async execute(query, params = [], method = 'all') {
        const res = await this.executeQuery(query, params, () => this.client.execute(query, params));
        if (method === 'get') {
            return res[0][0];
        }
        return res[0];
    }
    getConnectionOptions() {
        const ret = super.getConnectionOptions();
        if (this.config.get('multipleStatements')) {
            ret.multipleStatements = this.config.get('multipleStatements');
        }
        return ret;
    }
    async loadFile(path) {
        await this.client.query((await fs_extra_1.readFile(path)).toString());
    }
    async query(sql) {
        const now = Date.now();
        await this.client.query(sql);
        this.logQuery(sql, Date.now() - now);
    }
}
exports.MySqlConnection = MySqlConnection;
