"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pg_1 = require("pg");
const fs_extra_1 = require("fs-extra");
const Connection_1 = require("./Connection");
class PostgreSqlConnection extends Connection_1.Connection {
    async connect() {
        this.client = new pg_1.Client(this.getConnectionOptions());
        await this.client.connect();
    }
    async close(force) {
        await this.client.end();
    }
    async isConnected() {
        try {
            await this.client.query('SELECT 1');
            return true;
        }
        catch (_a) {
            return false;
        }
    }
    async beginTransaction(savepoint) {
        await this.execute(savepoint ? `SAVEPOINT ${savepoint}` : 'START TRANSACTION', [], 'run');
    }
    async commit(savepoint) {
        await this.execute(savepoint ? `RELEASE SAVEPOINT ${savepoint}` : 'COMMIT', [], 'run');
    }
    async rollback(savepoint) {
        await this.execute(savepoint ? `ROLLBACK TO SAVEPOINT ${savepoint}` : 'ROLLBACK', [], 'run');
    }
    getDefaultClientUrl() {
        return 'postgre://postgres@127.0.0.1:5432';
    }
    async execute(query, params = [], method = 'all') {
        const res = await this.executeQuery(query, params, () => this.client.query(query, params));
        return this.transformResult(res, method);
    }
    async loadFile(path) {
        await this.client.query((await fs_extra_1.readFile(path)).toString());
    }
    transformResult(res, method) {
        if (method === 'get') {
            return res.rows[0];
        }
        if (method === 'run') {
            return {
                affectedRows: res.rowCount || 0,
                insertId: res.rows[0] ? res.rows[0].id : 0,
                row: res.rows[0],
            };
        }
        return res.rows;
    }
}
exports.PostgreSqlConnection = PostgreSqlConnection;
