"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sqlite_1 = __importDefault(require("sqlite"));
const fs_extra_1 = require("fs-extra");
const Connection_1 = require("./Connection");
class SqliteConnection extends Connection_1.Connection {
    async connect() {
        this.client = await sqlite_1.default.open(this.config.get('dbName'));
        await this.client.exec('PRAGMA foreign_keys = ON');
    }
    async close(force) {
        await this.client.close();
    }
    async isConnected() {
        return this.client.driver.open;
    }
    async beginTransaction(savepoint) {
        await this.execute(savepoint ? `SAVEPOINT ${savepoint}` : 'BEGIN', [], 'run');
    }
    async commit(savepoint) {
        await this.execute(savepoint ? `RELEASE SAVEPOINT ${savepoint}` : 'COMMIT', [], 'run');
    }
    async rollback(savepoint) {
        await this.execute(savepoint ? `ROLLBACK TO SAVEPOINT ${savepoint}` : 'ROLLBACK', [], 'run');
    }
    getDefaultClientUrl() {
        return '';
    }
    getClientUrl() {
        return '';
    }
    async execute(query, params = [], method = 'all') {
        params = params.map(p => {
            if (p instanceof Date) {
                p = p.toISOString();
            }
            if (typeof p === 'boolean') {
                p = +p;
            }
            return p;
        });
        const res = await this.executeQuery(query, params, async () => {
            const statement = await this.client.prepare(query);
            const result = await statement[method](...params);
            await statement.finalize();
            return result;
        });
        return this.transformResult(res, method);
    }
    async loadFile(path) {
        await this.client.exec((await fs_extra_1.readFile(path)).toString());
    }
    transformResult(res, method) {
        if (method === 'run') {
            return {
                affectedRows: res.changes,
                insertId: res.lastID,
            };
        }
        return res;
    }
}
exports.SqliteConnection = SqliteConnection;
