import { EntityData, IEntityType, IPrimaryKey } from '../decorators';
import { DatabaseDriver } from './DatabaseDriver';
import { Connection, QueryResult } from '../connections';
import { FilterQuery } from './IDatabaseDriver';
import { QueryBuilder, QueryOrderMap } from '../query';
import { LockMode } from '../unit-of-work';
export declare abstract class AbstractSqlDriver<C extends Connection> extends DatabaseDriver<C> {
    find<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, populate?: string[], orderBy?: QueryOrderMap, limit?: number, offset?: number): Promise<T[]>;
    findOne<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | string, populate?: string[], orderBy?: QueryOrderMap, fields?: string[], lockMode?: LockMode): Promise<T | null>;
    count(entityName: string, where: any): Promise<number>;
    nativeInsert<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): Promise<QueryResult>;
    nativeUpdate<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, data: EntityData<T>): Promise<QueryResult>;
    nativeDelete<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | string | any): Promise<QueryResult>;
    protected createQueryBuilder(entityName: string): QueryBuilder;
    protected extractManyToMany<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): EntityData<T>;
    protected processManyToMany<T extends IEntityType<T>>(entityName: string, pk: IPrimaryKey, collections: EntityData<T>): Promise<void>;
}
