import { FilterQuery, IDatabaseDriver } from './IDatabaseDriver';
import { EntityData, EntityMetadata, EntityProperty, IEntity, IEntityType, IPrimaryKey } from '../decorators';
import { Connection, QueryResult } from '../connections';
import { Configuration } from '../utils';
import { QueryOrderMap } from '../query';
import { Platform } from '../platforms';
import { LockMode } from '../unit-of-work';
export declare abstract class DatabaseDriver<C extends Connection> implements IDatabaseDriver<C> {
    protected readonly config: Configuration;
    protected readonly connection: C;
    protected readonly platform: Platform;
    protected readonly metadata: Record<string, EntityMetadata<any>>;
    protected readonly logger: import("../utils").Logger;
    protected transactionLevel: number;
    protected transactionRolledBack: boolean;
    constructor(config: Configuration);
    abstract find<T extends IEntity>(entityName: string, where: FilterQuery<T>, populate?: string[], orderBy?: QueryOrderMap, limit?: number, offset?: number): Promise<T[]>;
    abstract findOne<T extends IEntity>(entityName: string, where: FilterQuery<T> | string, populate: string[], orderBy?: QueryOrderMap, fields?: string[], lockMode?: LockMode): Promise<T | null>;
    abstract nativeInsert<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): Promise<QueryResult>;
    abstract nativeUpdate<T extends IEntity>(entityName: string, where: FilterQuery<IEntity> | IPrimaryKey, data: EntityData<T>): Promise<QueryResult>;
    abstract nativeDelete<T extends IEntity>(entityName: string, where: FilterQuery<IEntity> | IPrimaryKey): Promise<QueryResult>;
    abstract count<T extends IEntity>(entityName: string, where: FilterQuery<T>): Promise<number>;
    aggregate(entityName: string, pipeline: any[]): Promise<any[]>;
    loadFromPivotTable<T extends IEntity>(prop: EntityProperty, owners: IPrimaryKey[]): Promise<Record<string, T[]>>;
    mapResult<T extends IEntityType<T>>(result: EntityData<T>, meta: EntityMetadata): T | null;
    getConnection(): C;
    beginTransaction(): Promise<void>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
    transactional(cb: () => Promise<any>): Promise<any>;
    isInTransaction(): boolean;
    getPlatform(): Platform;
    protected getPrimaryKeyField(entityName: string): string;
    private runTransaction;
    private getSavePointName;
}
