"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PostgreSqlConnection_1 = require("../connections/PostgreSqlConnection");
const AbstractSqlDriver_1 = require("./AbstractSqlDriver");
const query_1 = require("../query");
const PostgreSqlPlatform_1 = require("../platforms/PostgreSqlPlatform");
class PostgreSqlDriver extends AbstractSqlDriver_1.AbstractSqlDriver {
    constructor() {
        super(...arguments);
        this.connection = new PostgreSqlConnection_1.PostgreSqlConnection(this.config);
        this.platform = new PostgreSqlPlatform_1.PostgreSqlPlatform();
    }
    async nativeInsert(entityName, data) {
        const collections = this.extractManyToMany(entityName, data);
        const qb = this.createQueryBuilder(entityName).insert(data);
        const params = qb.getParams();
        let sql = qb.getQuery();
        if (qb.type === query_1.QueryType.INSERT && Object.keys(params).length === 0) {
            const pk = this.getPrimaryKeyField(entityName);
            const prop = this.metadata[entityName].properties[pk];
            sql = sql.replace('() VALUES ()', `("${prop.fieldName}") VALUES (DEFAULT)`);
        }
        const res = await this.connection.execute(sql, params, 'run');
        const pk = this.getPrimaryKeyField(entityName);
        res.insertId = res.insertId || data[pk];
        await this.processManyToMany(entityName, res.insertId, collections);
        return res;
    }
}
exports.PostgreSqlDriver = PostgreSqlDriver;
