"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const EntityTransformer_1 = require("./EntityTransformer");
const EntityAssigner_1 = require("./EntityAssigner");
class EntityHelper {
    static async init(entity, populated = true, lockMode) {
        await entity.__em.findOne(entity.constructor.name, entity.__primaryKey, { refresh: true, lockMode });
        entity.populated(populated);
        entity.__lazyInitialized = true;
        return entity;
    }
    static decorate(meta, em) {
        const pk = meta.properties[meta.primaryKey];
        if (pk.name === '_id' && !meta.properties.id) {
            EntityHelper.defineIdProperty(meta, em);
        }
        EntityHelper.defineBaseProperties(meta, em);
        EntityHelper.definePrimaryKeyProperties(meta);
        if (!meta.prototype.assign) { // assign can be overridden
            meta.prototype.assign = function (data, options) {
                EntityAssigner_1.EntityAssigner.assign(this, data, options);
            };
        }
        if (!meta.prototype.toJSON) { // toJSON can be overridden
            meta.prototype.toJSON = function (...args) {
                return EntityTransformer_1.EntityTransformer.toObject(this, ...args.slice(meta.toJsonParams.length));
            };
        }
    }
    /**
     * defines magic id property getter/setter if PK property is `_id` and there is no `id` property defined
     */
    static defineIdProperty(meta, em) {
        Object.defineProperty(meta.prototype, 'id', {
            get() {
                return this._id ? em.getDriver().getPlatform().normalizePrimaryKey(this._id) : null;
            },
            set(id) {
                this._id = id ? em.getDriver().getPlatform().denormalizePrimaryKey(id) : null;
            },
        });
    }
    static defineBaseProperties(meta, em) {
        Object.defineProperties(meta.prototype, {
            __populated: { value: false, writable: true },
            __lazyInitialized: { value: false, writable: true },
            __entity: { value: true },
            __em: { value: em },
            __uuid: {
                get() {
                    if (!this.___uuid) {
                        Object.defineProperty(this, '___uuid', { value: uuid_1.v4() });
                    }
                    return this.___uuid;
                },
            },
        });
        meta.prototype.isInitialized = function () {
            return this.__initialized !== false;
        };
        meta.prototype.populated = function (populated = true) {
            this.__populated = populated;
        };
        meta.prototype.toObject = function (ignoreFields = []) {
            return EntityTransformer_1.EntityTransformer.toObject(this, ignoreFields);
        };
        meta.prototype.init = function (populated = true) {
            return EntityHelper.init(this, populated);
        };
    }
    static definePrimaryKeyProperties(meta) {
        Object.defineProperties(meta.prototype, {
            __primaryKeyField: { value: meta.primaryKey },
            __primaryKey: {
                get() {
                    return this[meta.primaryKey];
                },
                set(id) {
                    this[meta.primaryKey] = id;
                },
            },
            __serializedPrimaryKeyField: { value: meta.serializedPrimaryKey },
            __serializedPrimaryKey: {
                get() {
                    return this[meta.serializedPrimaryKey];
                },
            },
        });
    }
}
exports.EntityHelper = EntityHelper;
