import { EntityManager, FindOneOptions, FindOptions } from '../EntityManager';
import { EntityData, EntityName, IEntity, IEntityType, IPrimaryKey } from '../decorators';
import { QueryBuilder, QueryOrderMap } from '../query';
import { FilterQuery } from '..';
export declare class EntityRepository<T extends IEntityType<T>> {
    private readonly em;
    protected readonly entityName: EntityName<T>;
    constructor(em: EntityManager, entityName: EntityName<T>);
    persist(entity: T | IEntity[], flush?: boolean): Promise<void>;
    persistAndFlush(entity: IEntity | IEntity[]): Promise<void>;
    persistLater(entity: IEntity | IEntity[]): void;
    createQueryBuilder(alias?: string): QueryBuilder;
    findOne(where: FilterQuery<T> | IPrimaryKey, populate?: string[], orderBy?: QueryOrderMap): Promise<T | null>;
    findOne(where: FilterQuery<T> | IPrimaryKey, populate?: FindOneOptions, orderBy?: QueryOrderMap): Promise<T | null>;
    find(where: FilterQuery<T> | IPrimaryKey, options?: FindOptions): Promise<T[]>;
    find(where: FilterQuery<T> | IPrimaryKey, populate?: string[], orderBy?: QueryOrderMap, limit?: number, offset?: number): Promise<T[]>;
    findAll(options?: FindOptions): Promise<T[]>;
    findAll(populate?: string[], orderBy?: QueryOrderMap, limit?: number, offset?: number): Promise<T[]>;
    remove(where: T | FilterQuery<T> | IPrimaryKey, flush?: boolean): Promise<number>;
    removeAndFlush(entity: IEntity): Promise<void>;
    removeLater(entity: IEntity): void;
    flush(): Promise<void>;
    nativeInsert(data: EntityData<T>): Promise<IPrimaryKey>;
    nativeUpdate(where: FilterQuery<T>, data: EntityData<T>): Promise<number>;
    nativeDelete(where: FilterQuery<T> | any): Promise<number>;
    map(result: EntityData<T>): T;
    aggregate(pipeline: any[]): Promise<any[]>;
    /**
     * Gets a reference to the entity identified by the given type and identifier without actually loading it, if the entity is not yet loaded
     */
    getReference(id: IPrimaryKey): T;
    canPopulate(property: string): boolean;
    /**
     * Creates new instance of given entity and populates it with given data
     */
    create(data: EntityData<T>): T;
    count(where?: FilterQuery<T>): Promise<number>;
}
