"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const ArrayCollection_1 = require("./ArrayCollection");
const metadata_1 = require("../metadata");
class EntityTransformer {
    static toObject(entity, ignoreFields = []) {
        const platform = entity.__em.getDriver().getPlatform();
        const pk = platform.getSerializedPrimaryKeyField(entity.__primaryKeyField);
        const meta = metadata_1.MetadataStorage.getMetadata(entity.constructor.name);
        const pkProp = meta.properties[meta.primaryKey];
        const ret = (entity.__primaryKey && !pkProp.hidden ? { [pk]: platform.normalizePrimaryKey(entity.__primaryKey) } : {});
        if (!entity.isInitialized() && entity.__primaryKey) {
            return ret;
        }
        Object.keys(entity)
            .filter(prop => this.isVisible(meta, prop, entity, ignoreFields))
            .map(prop => [prop, EntityTransformer.processProperty(prop, entity, ignoreFields)])
            .filter(([, value]) => typeof value !== 'undefined')
            .forEach(([prop, value]) => ret[prop] = value);
        return ret;
    }
    static isVisible(meta, prop, entity, ignoreFields) {
        const hidden = meta.properties[prop] && !meta.properties[prop].hidden;
        return hidden && prop !== entity.__primaryKeyField && !prop.startsWith('_') && !ignoreFields.includes(prop);
    }
    static processProperty(prop, entity, ignoreFields) {
        if (entity[prop] instanceof ArrayCollection_1.ArrayCollection) {
            return EntityTransformer.processCollection(prop, entity);
        }
        if (utils_1.Utils.isEntity(entity[prop])) {
            return EntityTransformer.processEntity(prop, entity, ignoreFields);
        }
        return entity[prop];
    }
    static processEntity(prop, entity, ignoreFields) {
        const child = entity[prop];
        const platform = child.__em.getDriver().getPlatform();
        if (child.isInitialized() && child.__populated && child !== entity && !child.__lazyInitialized) {
            const meta = metadata_1.MetadataStorage.getMetadata(child.constructor.name);
            const args = [...meta.toJsonParams.map(() => undefined), ignoreFields];
            return child.toJSON(...args);
        }
        return platform.normalizePrimaryKey(child.__primaryKey);
    }
    static processCollection(prop, entity) {
        const col = entity[prop];
        if (col.isInitialized(true) && col.shouldPopulate()) {
            return col.toArray();
        }
        if (col.isInitialized()) {
            return col.getIdentifiers();
        }
    }
}
exports.EntityTransformer = EntityTransformer;
