"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_morph_1 = require("ts-morph");
const fs_extra_1 = require("fs-extra");
const MetadataProvider_1 = require("./MetadataProvider");
const utils_1 = require("../utils");
class TypeScriptMetadataProvider extends MetadataProvider_1.MetadataProvider {
    constructor() {
        super(...arguments);
        this.project = new ts_morph_1.Project();
    }
    async loadEntityMetadata(meta, name) {
        if (!meta.path) {
            return;
        }
        await this.initProperties(meta, name);
    }
    async initProperties(meta, name) {
        // init types and column names
        for (const prop of Object.values(meta.properties)) {
            if (prop.entity) {
                prop.type = utils_1.Utils.className(prop.entity());
            }
            else if (!prop.type) {
                const file = meta.path.match(/\/[^\/]+$/)[0].replace(/\.js$/, '.ts');
                prop.type = await this.readTypeFromSource(file, name, prop);
            }
        }
    }
    async readTypeFromSource(file, name, prop) {
        const source = await this.getSourceFile(file);
        const properties = source.getClass(name).getInstanceProperties();
        const property = properties.find(v => v.getName() === prop.name);
        return property.getType().getText(property);
    }
    async getSourceFile(file) {
        if (!this.sources) {
            await this.initSourceFiles();
        }
        const source = this.sources.find(s => s.getFilePath().endsWith(file));
        if (!source) {
            throw new Error(`Source file for entity ${file} not found, check your 'entitiesDirsTs' option`);
        }
        return source;
    }
    async initSourceFiles() {
        const tsDirs = this.config.get('entitiesDirsTs');
        if (tsDirs.length > 0) {
            const dirs = await this.validateDirectories(tsDirs);
            this.sources = this.project.addExistingSourceFiles(dirs);
        }
        else {
            this.sources = this.project.addSourceFilesFromTsConfig(this.config.get('tsConfigPath'));
        }
    }
    async validateDirectories(dirs) {
        const ret = [];
        for (const dir of dirs) {
            const path = utils_1.Utils.normalizePath(this.config.get('baseDir'), dir);
            if (!await fs_extra_1.pathExists(path)) {
                throw new Error(`Path ${path} does not exist`);
            }
            ret.push(utils_1.Utils.normalizePath(path, '**', '*.ts'));
        }
        return ret;
    }
}
exports.TypeScriptMetadataProvider = TypeScriptMetadataProvider;
