import { NamingStrategy } from '../naming-strategy';
import { IPrimaryKey } from '../decorators';
import { SchemaHelper } from '../schema';
export declare abstract class Platform {
    protected abstract schemaHelper: SchemaHelper;
    usesPivotTable(): boolean;
    supportsTransactions(): boolean;
    supportsSavePoints(): boolean;
    getNamingStrategy(): {
        new (): NamingStrategy;
    };
    getParameterPlaceholder(index?: number): string;
    usesReturningStatement(): boolean;
    usesCascadeStatement(): boolean;
    getSchemaHelper(): SchemaHelper;
    requiresNullableForAlteringColumn(): boolean;
    /**
     * Normalizes primary key wrapper to scalar value (e.g. mongodb's ObjectID to string)
     */
    normalizePrimaryKey<T = number | string>(data: IPrimaryKey): T;
    /**
     * Converts scalar primary key representation to native driver wrapper (e.g. string to mongodb's ObjectID)
     */
    denormalizePrimaryKey(data: IPrimaryKey): IPrimaryKey;
    /**
     * Used when serializing via toObject and toJSON methods, allows to use different PK field name (like `id` instead of `_id`)
     */
    getSerializedPrimaryKeyField(field: string): string;
    /**
     * Returns the SQL specific for the platform to get the current timestamp
     */
    getCurrentTimestampSQL(length: number): string;
    /**
     * Returns the FOR UPDATE expression.
     *
     */
    getForUpdateSQL(): string;
    /**
     * Returns the SQL snippet to append to any SELECT statement which locks rows in shared read lock.
     *
     * This defaults to the ANSI SQL "FOR UPDATE", which is an exclusive lock (Write). Some database
     * vendors allow to lighten this constraint up to be a real read lock.
     */
    getReadLockSQL(): string;
    /**
     * Returns the SQL snippet to append to any SELECT statement which obtains an exclusive lock on the rows.
     *
     * The semantics of this lock mode should equal the SELECT .. FOR UPDATE of the ANSI SQL standard.
     */
    getWriteLockSQL(): string;
}
