"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const naming_strategy_1 = require("../naming-strategy");
class Platform {
    usesPivotTable() {
        return true;
    }
    supportsTransactions() {
        return true;
    }
    supportsSavePoints() {
        return false;
    }
    getNamingStrategy() {
        return naming_strategy_1.UnderscoreNamingStrategy;
    }
    getParameterPlaceholder(index) {
        return '?';
    }
    usesReturningStatement() {
        return false;
    }
    usesCascadeStatement() {
        return false;
    }
    getSchemaHelper() {
        return this.schemaHelper;
    }
    requiresNullableForAlteringColumn() {
        return false;
    }
    /**
     * Normalizes primary key wrapper to scalar value (e.g. mongodb's ObjectID to string)
     */
    normalizePrimaryKey(data) {
        return data;
    }
    /**
     * Converts scalar primary key representation to native driver wrapper (e.g. string to mongodb's ObjectID)
     */
    denormalizePrimaryKey(data) {
        return data;
    }
    /**
     * Used when serializing via toObject and toJSON methods, allows to use different PK field name (like `id` instead of `_id`)
     */
    getSerializedPrimaryKeyField(field) {
        return field;
    }
    /**
     * Returns the SQL specific for the platform to get the current timestamp
     */
    getCurrentTimestampSQL(length) {
        return 'CURRENT_TIMESTAMP' + (length ? `(${length})` : '');
    }
    /**
     * Returns the FOR UPDATE expression.
     *
     */
    getForUpdateSQL() {
        return 'FOR UPDATE';
    }
    /**
     * Returns the SQL snippet to append to any SELECT statement which locks rows in shared read lock.
     *
     * This defaults to the ANSI SQL "FOR UPDATE", which is an exclusive lock (Write). Some database
     * vendors allow to lighten this constraint up to be a real read lock.
     */
    getReadLockSQL() {
        return this.getForUpdateSQL();
    }
    /**
     * Returns the SQL snippet to append to any SELECT statement which obtains an exclusive lock on the rows.
     *
     * The semantics of this lock mode should equal the SELECT .. FOR UPDATE of the ANSI SQL standard.
     */
    getWriteLockSQL() {
        return this.getForUpdateSQL();
    }
}
exports.Platform = Platform;
