import { QueryBuilderHelper } from './QueryBuilderHelper';
import { EntityMetadata, EntityProperty } from '../decorators';
import { QueryOrderMap, QueryType } from './enums';
import { IDatabaseDriver } from '../drivers';
import { LockMode } from '../unit-of-work';
/**
 * SQL query builder
 */
export declare class QueryBuilder {
    private readonly entityName;
    private readonly metadata;
    private readonly driver;
    readonly alias: string;
    type: QueryType;
    private aliasCounter;
    private flags;
    private finalized;
    private _fields;
    private _populate;
    private _populateMap;
    private _joins;
    private _aliasMap;
    private _cond;
    private _data;
    private _orderBy;
    private _groupBy;
    private _having;
    private _limit;
    private _offset;
    private lockMode?;
    private readonly connection;
    private readonly platform;
    private readonly helper;
    constructor(entityName: string, metadata: Record<string, EntityMetadata>, driver: IDatabaseDriver, alias?: string);
    select(fields: string | string[], distinct?: boolean): this;
    insert(data: any): this;
    update(data: any): this;
    delete(cond?: any): this;
    truncate(): this;
    count(field?: string, distinct?: boolean): this;
    join(field: string, alias: string, type?: 'left' | 'inner'): this;
    leftJoin(field: string, alias: string): this;
    where(cond: Record<string, any>, operator?: keyof typeof QueryBuilderHelper.GROUP_OPERATORS): this;
    where(cond: string, params?: any[], operator?: keyof typeof QueryBuilderHelper.GROUP_OPERATORS): this;
    andWhere(cond: Record<string, any>): this;
    andWhere(cond: string, params?: any[]): this;
    orWhere(cond: Record<string, any>): this;
    orWhere(cond: string, params?: any[]): this;
    orderBy(orderBy: QueryOrderMap): this;
    groupBy(fields: string | string[]): this;
    having(cond: Record<string, any> | string, params?: any[]): this;
    populate(populate: string[]): this;
    limit(limit: number, offset?: number): this;
    offset(offset: number): this;
    setLockMode(mode?: LockMode): this;
    getQuery(): string;
    getParams(): any[];
    execute(method?: 'all' | 'get' | 'run', mapResults?: boolean): Promise<any>;
    clone(): QueryBuilder;
    private prepareFields;
    private processWhere;
    private processOneToOne;
    private processManyToMany;
    private processOneToMany;
    private init;
    private getQueryBase;
    private finalize;
}
export interface JoinOptions {
    table: string;
    type: 'left' | 'inner';
    alias: string;
    ownerAlias: string;
    joinColumn?: string;
    inverseJoinColumn?: string;
    primaryKey?: string;
    prop: EntityProperty;
}
