"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SchemaHelper_1 = require("./SchemaHelper");
class PostgreSqlSchemaHelper extends SchemaHelper_1.SchemaHelper {
    getSchemaBeginning() {
        return `SET NAMES 'utf8';\nSET session_replication_role = 'replica';\n\n\n`;
    }
    getSchemaEnd() {
        return `SET session_replication_role = 'origin';\n`;
    }
    getAutoIncrementStatement(meta) {
        return `DEFAULT NEXTVAL('${meta.collection}_seq')`;
    }
    getTypeDefinition(prop) {
        return super.getTypeDefinition(prop, PostgreSqlSchemaHelper.TYPES, PostgreSqlSchemaHelper.DEFAULT_TYPE_LENGTHS);
    }
    getUnsignedSuffix(prop) {
        return ` check (${this.quoteIdentifier(prop.fieldName)} > 0)`;
    }
    supportsSequences() {
        return true;
    }
    indexForeignKeys() {
        return false;
    }
    dropTable(meta) {
        let ret = `DROP TABLE IF EXISTS ${this.quoteIdentifier(meta.collection)} CASCADE;\n`;
        ret += `DROP SEQUENCE IF EXISTS ${this.quoteIdentifier(meta.collection + '_seq')};\n`;
        return ret;
    }
}
PostgreSqlSchemaHelper.TYPES = {
    number: 'int',
    float: 'float',
    double: 'double precision',
    string: 'varchar(?)',
    date: 'timestamp(?)',
    boolean: 'boolean',
    text: 'text',
    json: 'json',
};
PostgreSqlSchemaHelper.DEFAULT_TYPE_LENGTHS = {
    string: 255,
    date: 0,
};
exports.PostgreSqlSchemaHelper = PostgreSqlSchemaHelper;
