"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class SchemaHelper {
    getIdentifierQuoteCharacter() {
        return '"';
    }
    getSchemaBeginning() {
        return '';
    }
    getSchemaEnd() {
        return '';
    }
    getSchemaTableEnd() {
        return '';
    }
    getAutoIncrementStatement(meta) {
        return 'AUTO_INCREMENT';
    }
    getPrimaryKeySubtype(meta) {
        return 'NOT NULL';
    }
    getTypeDefinition(prop, types = {}, lengths = {}) {
        const t = prop.type.toLowerCase();
        let type = types[t] || types.json || types.text || t;
        if (type.includes('(?)')) {
            const length = prop.length || lengths[t];
            type = length ? type.replace('?', length) : type.replace('(?)', '');
        }
        return type;
    }
    getUnsignedSuffix(prop) {
        return '';
    }
    supportsSchemaConstraints() {
        return true;
    }
    supportsSchemaMultiAlter() {
        return true;
    }
    supportsSequences() {
        return false;
    }
    quoteIdentifier(field) {
        const quoteChar = this.getIdentifierQuoteCharacter();
        return quoteChar + field + quoteChar;
    }
    dropTable(meta) {
        const pkProp = meta.properties[meta.primaryKey];
        let ret = `DROP TABLE IF EXISTS ${this.quoteIdentifier(meta.collection)};\n`;
        if (this.supportsSequences() && pkProp.type === 'number') {
            ret += `DROP SEQUENCE IF EXISTS ${this.quoteIdentifier(meta.collection + '_seq')};\n`;
        }
        return ret;
    }
    indexForeignKeys() {
        return true;
    }
    createPrimaryKeyColumn(meta, prop) {
        let ret = ' ' + this.getPrimaryKeySubtype(meta);
        if (prop.type === 'number') {
            ret += ' ' + this.getAutoIncrementStatement(meta);
        }
        return ret;
    }
    createColumn(meta, prop, nullable) {
        let ret = '';
        if (prop.unique) {
            ret += ' UNIQUE';
        }
        if (!nullable) {
            ret += ' NOT NULL';
        }
        // support falsy default values like `0`, `false or empty string
        if (typeof prop.default !== 'undefined') {
            return ret + ` DEFAULT ${prop.default}`;
        }
        if (nullable) {
            return ret + ` DEFAULT NULL`;
        }
        return ret;
    }
}
exports.SchemaHelper = SchemaHelper;
